﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using System;
using System.IO;
using VoxelRendererQuery.Includes;
using VoxelRendererQuery.Raytracer;
using VoxelRendererQuery.Ressources;

namespace TestVoxelRenderQuery
{
    public class Game1 : Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public Game1()
        {
            Content.RootDirectory = "Content";

            graphics = new GraphicsDeviceManager(this);
            graphics.GraphicsProfile = GraphicsProfile.HiDef;
            graphics.IsFullScreen = false;


        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        [VoxelDefinition]
        struct Voxel
        {
            [SizeInBits(8)]
            public int r;
            [SizeInBits(8)]
            public int g;
            [SizeInBits(8)]
            public int b;

        }

        RTvoXelQuery<Voxel> rtx;
        Texture2D tex;
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            IsFixedTimeStep = false;

            rtx = new RTvoXelQuery<Voxel>(GraphicsDevice, Content)
            {
                RTvoXelSource = File.ReadAllText(@"example.nhlsl")
            };
            rtx.Compile(RTvoXelQuery<Voxel>.CompilerProfile.DirectX_11);

            YawPitchCamera.Width = GraphicsDevice.Viewport.Width;
            YawPitchCamera.Height = GraphicsDevice.Viewport.Height;

            YawPitchCamera.Initialize();

            rtx.Volume = createSphere(64);

            tex = Content.Load<Texture2D>("test");
            rtx.Parameters["VoxelTextures"].SetValue(tex);

            base.LoadContent();
        }

        private VoxelVolume3D<Voxel> createSphere(int radius)
        {
            VoxelVolume3D<Voxel> voxels = new VoxelVolume3D<Voxel>(rtx, radius * 2, radius * 2, radius * 2);
            Voxel[] data = new Voxel[voxels.Width * voxels.Height * voxels.Depth];

            Random random = new Random();

            Vector3 c = new Vector3(radius);
            for (int x = 0; x < voxels.Width; x++)
            {
                for (int z = 0; z < voxels.Depth; z++)
                {
                    for (int y = 0; y < voxels.Height; y++)
                    {
                        float r = (new Vector3(x, y, z) - c).LengthSquared();
                        if (r <= radius * radius)
                        {
                            data[x + z * voxels.Width + y * voxels.Width * voxels.Depth] = new Voxel()
                            {
                                r = random.Next(0, 255),
                                g = random.Next(0, 255),
                                b = random.Next(0, 255),

                            };
                        }
                    }
                }
            }

            voxels.SetVoxelData(data);

            data = new Voxel[0];
            return voxels;
        }

        protected override void Update(GameTime gameTime)
        {
            if (!IsActive)
                return;


            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                Exit();


            if (Keyboard.GetState().IsKeyDown(Keys.W))
                YawPitchCamera.Move(new Vector3(0, 0, 1) * (float)gameTime.ElapsedGameTime.TotalSeconds);


            if (Keyboard.GetState().IsKeyDown(Keys.A))
                YawPitchCamera.Move(new Vector3(-1, 0, 0) * (float)gameTime.ElapsedGameTime.TotalSeconds);


            if (Keyboard.GetState().IsKeyDown(Keys.S))
                YawPitchCamera.Move(new Vector3(0, 0, -1) * (float)gameTime.ElapsedGameTime.TotalSeconds);


            if (Keyboard.GetState().IsKeyDown(Keys.D))
                YawPitchCamera.Move(new Vector3(1, 0, 0) * (float)gameTime.ElapsedGameTime.TotalSeconds);

            if (Keyboard.GetState().IsKeyDown(Keys.Space))
                YawPitchCamera.Move(new Vector3(0, 1, 0) * (float)gameTime.ElapsedGameTime.TotalSeconds);


            if (Keyboard.GetState().IsKeyDown(Keys.LeftControl))
                YawPitchCamera.Move(new Vector3(0, -1, 0) * (float)gameTime.ElapsedGameTime.TotalSeconds);



            YawPitchCamera.Update();

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            rtx.Parameters["screenWidth"].SetValue(1.0f / (float)GraphicsDevice.Viewport.Width);
            rtx.Parameters["screenHeight"].SetValue(1.0f / (float)GraphicsDevice.Viewport.Height);
            rtx.Parameters["aspectRatio"].SetValue(new Vector2(GraphicsDevice.Viewport.AspectRatio, 1));


            rtx.Parameters["cameraRotation"].SetValue(YawPitchCamera.YawPitchMatrix);
            rtx.Parameters["cameraPosition"].SetValue(YawPitchCamera.CameraPosition);


            float fov = 2.0f * .5f;
            float focalDistance = 1.0f / MathF.Tan(fov * 0.5f);

            rtx.Parameters["focalDistance"].SetValue(focalDistance);

            rtx.Draw(spriteBatch);
            base.Draw(gameTime);
        }

    }
}
